@@ -30,10 +30,14 @@
 #include "channel.h"
 
 int agentreq(struct ChanSess * chansess);
-int agentaccept(struct ChanSess * chansess);
-void agentcleanup(struct ChanSess * chansess);
 void agentsetauth(struct ChanSess *chansess);
+void agentcleanup(struct ChanSess * chansess);
 void agentset(struct ChanSess *chansess);
 
+#ifdef __hpux
+#define seteuid(a)       setresuid(-1, (a), -1)
+#define setegid(a)       setresgid(-1, (a), -1)
+#endif
+
 #endif /* DROPBEAR_AGENTFWD */
 #endif /* _AGENTFWD_H_ */
