@@ -27,7 +27,6 @@
 #define _ALGO_H_
 
 #include "includes.h"
-#include "util.h"
 #include "buffer.h"
 
 struct Algo_Type {
@@ -52,20 +51,24 @@ extern const struct dropbear_cipher dropbear_nocipher;
 extern const struct dropbear_hash dropbear_nohash;
 
 struct dropbear_cipher {
-	const struct _cipher_descriptor *cipherdesc;
+	const struct ltc_cipher_descriptor *cipherdesc;
 	unsigned long keysize;
 	unsigned char blocksize;
 };
 
 struct dropbear_hash {
-	const struct _hash_descriptor *hashdesc;
+	const struct ltc_hash_descriptor *hashdesc;
 	unsigned long keysize;
 	unsigned char hashsize;
 };
 
 void crypto_init();
-int have_algo(char* algo, int algolen, algo_type algos[]);
-algo_type * buf_match_algo(buffer* buf, algo_type localalgos[]);
+int have_algo(char* algo, size_t algolen, algo_type algos[]);
 void buf_put_algolist(buffer * buf, algo_type localalgos[]);
 
+algo_type * svr_buf_match_algo(buffer* buf, algo_type localalgos[],
+		int *goodguess);
+algo_type * cli_buf_match_algo(buffer* buf, algo_type localalgos[],
+		int *goodguess);
+
 #endif /* _ALGO_H_ */
