@@ -1,3 +1,5 @@
+#include <tommath.h>
+#ifdef BN_MP_DIV_2_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
  * LibTomMath is a library that provides multiple-precision
@@ -10,13 +12,11 @@
  * The library is free for all purposes without any express
  * guarantee it works.
  *
- * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
+ * Tom St Denis, tomstdenis@gmail.com, http://math.libtomcrypt.com
  */
-#include <tommath.h>
 
 /* b = a/2 */
-int
-mp_div_2 (mp_int * a, mp_int * b)
+int mp_div_2(mp_int * a, mp_int * b)
 {
   int     x, res, oldused;
 
@@ -61,3 +61,8 @@ mp_div_2 (mp_int * a, mp_int * b)
   mp_clamp (b);
   return MP_OKAY;
 }
+#endif
+
+/* $Source: /cvs/libtom/libtommath/bn_mp_div_2.c,v $ */
+/* $Revision: 1.3 $ */
+/* $Date: 2006/03/31 14:18:44 $ */
