@@ -36,8 +36,6 @@
 #include "channel.h"
 
 static void read_packet_init();
-static void process_postauth_packet(unsigned int type);
-static void recv_unimplemented();
 static void writemac(buffer * outputbuffer, buffer * clearwritebuf);
 static int checkmac(buffer* hashbuf, buffer* readbuf);
 
@@ -52,42 +50,45 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len);
 void write_packet() {
 
 	int len, written;
-	buffer * writebuf;
+	buffer * writebuf = NULL;
 	
-	TRACE(("enter write_packet"));
-	assert(!isempty(&ses.writequeue));
+	TRACE(("enter write_packet"))
+	dropbear_assert(!isempty(&ses.writequeue));
 
 	/* Get the next buffer in the queue of encrypted packets to write*/
 	writebuf = (buffer*)examine(&ses.writequeue);
 
 	len = writebuf->len - writebuf->pos;
-	assert(len > 0);
+	dropbear_assert(len > 0);
 	/* Try to write as much as possible */
 	written = write(ses.sock, buf_getptr(writebuf, len), len);
 
 	if (written < 0) {
 		if (errno == EINTR) {
-			TRACE(("leave writepacket: EINTR"));
+			TRACE(("leave writepacket: EINTR"))
 			return;
 		} else {
 			dropbear_exit("error writing");
 		}
 	} 
+	
+	ses.last_packet_time = time(NULL);
 
 	if (written == 0) {
-		session_remoteclosed();
+		ses.remoteclosed();
 	}
 
 	if (written == len) {
 		/* We've finished with the packet, free it */
 		dequeue(&ses.writequeue);
 		buf_free(writebuf);
+		writebuf = NULL;
 	} else {
 		/* More packet left to write, leave it in the queue for later */
 		buf_incrpos(writebuf, written);
 	}
 
-	TRACE(("leave write_packet"));
+	TRACE(("leave write_packet"))
 }
 
 /* Non-blocking function reading available portion of a packet into the
@@ -99,7 +100,7 @@ void read_packet() {
 	unsigned int maxlen;
 	unsigned char blocksize;
 
-	TRACE(("enter read_packet"));
+	TRACE(("enter read_packet"))
 	blocksize = ses.keys->recv_algo_crypt->blocksize;
 	
 	if (ses.readbuf == NULL || ses.readbuf->len < blocksize) {
@@ -112,38 +113,39 @@ void read_packet() {
 		/* If we don't have the length of decryptreadbuf, we didn't read
 		 * a whole blocksize and should exit */
 		if (ses.decryptreadbuf->len == 0) {
-			TRACE(("leave read_packet: packetinit done"));
+			TRACE(("leave read_packet: packetinit done"))
 			return;
 		}
 	}
 
 	/* Attempt to read the remainder of the packet, note that there
 	 * mightn't be any available (EAGAIN) */
-	assert(ses.readbuf != NULL);
+	dropbear_assert(ses.readbuf != NULL);
 	maxlen = ses.readbuf->len - ses.readbuf->pos;
 	len = read(ses.sock, buf_getptr(ses.readbuf, maxlen), maxlen);
-	buf_incrpos(ses.readbuf, len);
 
 	if (len == 0) {
-		session_remoteclosed();
+		ses.remoteclosed();
 	}
 
 	if (len < 0) {
 		if (errno == EINTR || errno == EAGAIN) {
-			TRACE(("leave read_packet: EINTR or EAGAIN"));
+			TRACE(("leave read_packet: EINTR or EAGAIN"))
 			return;
 		} else {
 			dropbear_exit("error reading: %s", strerror(errno));
 		}
 	}
 
+	buf_incrpos(ses.readbuf, len);
+
 	if ((unsigned int)len == maxlen) {
 		/* The whole packet has been read */
 		decrypt_packet();
-		/* The main select() loop in session.h will process_packet() to
+		/* The main select() loop process_packet() to
 		 * handle the packet contents... */
 	}
-	TRACE(("leave read_packet"));
+	TRACE(("leave read_packet"))
 }
 
 /* Function used to read the initial portion of a packet, and determine the
@@ -162,7 +164,7 @@ static void read_packet_init() {
 	if (ses.readbuf == NULL) {
 		/* start of a new packet */
 		ses.readbuf = buf_new(INIT_READBUF);
-		assert(ses.decryptreadbuf == NULL);
+		dropbear_assert(ses.decryptreadbuf == NULL);
 		ses.decryptreadbuf = buf_new(blocksize);
 	}
 
@@ -172,11 +174,11 @@ static void read_packet_init() {
 	len = read(ses.sock, buf_getwriteptr(ses.readbuf, maxlen),
 			maxlen);
 	if (len == 0) {
-		session_remoteclosed();
+		ses.remoteclosed();
 	}
 	if (len < 0) {
 		if (errno == EINTR) {
-			TRACE(("leave read_packet_init: EINTR"));
+			TRACE(("leave read_packet_init: EINTR"))
 			return;
 		}
 		dropbear_exit("error reading: %s", strerror(errno));
@@ -201,6 +203,7 @@ static void read_packet_init() {
 		/* decrypt it */
 		if (cbc_decrypt(buf_getptr(ses.readbuf, blocksize), 
 					buf_getwriteptr(ses.decryptreadbuf,blocksize),
+					blocksize,
 					&ses.keys->recv_symmetric_struct) != CRYPT_OK) {
 			dropbear_exit("error decrypting");
 		}
@@ -211,10 +214,10 @@ static void read_packet_init() {
 	buf_setpos(ses.readbuf, blocksize);
 
 	/* check packet length */
-	if ((len > MAX_PACKET_LEN) ||
+	if ((len > RECV_MAX_PACKET_LEN) ||
 		(len < MIN_PACKET_LEN + macsize) ||
 		((len - macsize) % blocksize != 0)) {
-		dropbear_exit("bad packet size");
+		dropbear_exit("bad packet size %d", len);
 	}
 
 	buf_resize(ses.readbuf, len);
@@ -230,7 +233,7 @@ void decrypt_packet() {
 	unsigned int padlen;
 	unsigned int len;
 
-	TRACE(("enter decrypt_packet"));
+	TRACE(("enter decrypt_packet"))
 	blocksize = ses.keys->recv_algo_crypt->blocksize;
 	macsize = ses.keys->recv_algo_mac->hashsize;
 
@@ -254,6 +257,7 @@ void decrypt_packet() {
 		while (ses.readbuf->pos < ses.readbuf->len - macsize) {
 			if (cbc_decrypt(buf_getptr(ses.readbuf, blocksize), 
 						buf_getwriteptr(ses.decryptreadbuf, blocksize),
+						blocksize,
 						&ses.keys->recv_symmetric_struct) != CRYPT_OK) {
 				dropbear_exit("error decrypting");
 			}
@@ -279,7 +283,7 @@ void decrypt_packet() {
 	/* payload length */
 	/* - 4 - 1 is for LEN and PADLEN values */
 	len = ses.decryptreadbuf->len - padlen - 4 - 1;
-	if ((len > MAX_PAYLOAD_LEN) || (len < 1)) {
+	if ((len > RECV_MAX_PAYLOAD_LEN) || (len < 1)) {
 		dropbear_exit("bad packet size");
 	}
 
@@ -305,21 +309,20 @@ void decrypt_packet() {
 
 	ses.recvseq++;
 
-	TRACE(("leave decrypt_packet"));
+	TRACE(("leave decrypt_packet"))
 }
 
 /* Checks the mac in hashbuf, for the data in readbuf.
  * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int checkmac(buffer* macbuf, buffer* sourcebuf) {
 
-	unsigned char macsize;
+	unsigned int macsize;
 	hmac_state hmac;
 	unsigned char tempbuf[MAX_MAC_LEN];
-	unsigned long hashsize;
-	int len;
+	unsigned long bufsize;
+	unsigned int len;
 
 	macsize = ses.keys->recv_algo_mac->hashsize;
-
 	if (macsize == 0) {
 		return DROPBEAR_SUCCESS;
 	}
@@ -345,8 +348,8 @@ static int checkmac(buffer* macbuf, buffer* sourcebuf) {
 		dropbear_exit("HMAC error");
 	}
 
-	hashsize = sizeof(tempbuf);
-	if (hmac_done(&hmac, tempbuf, &hashsize) != CRYPT_OK) {
+	bufsize = sizeof(tempbuf);
+	if (hmac_done(&hmac, tempbuf, &bufsize) != CRYPT_OK) {
 		dropbear_exit("HMAC error");
 	}
 
@@ -402,171 +405,61 @@ static buffer* buf_decompress(buffer* buf, unsigned int len) {
 #endif
 
 
-/* process a decrypted packet, call the appropriate handler */
-void process_packet() {
-
-	unsigned char type;
-
-	TRACE(("enter process_packet"));
-
-	type = buf_getbyte(ses.payload);
-	TRACE(("process_packet: packet type = %d", type));
-
-	/* these packets we can receive at any time, regardless of expecting
-	 * other packets: */
-	switch(type) {
-
-		case SSH_MSG_IGNORE:
-		case SSH_MSG_DEBUG:
-			TRACE(("received SSH_MSG_IGNORE or SSH_MSG_DEBUG"));
-			goto out;
-
-		case SSH_MSG_UNIMPLEMENTED:
-			/* debugging XXX */
-			TRACE(("SSH_MSG_UNIMPLEMENTED"));
-			dropbear_exit("received SSH_MSG_UNIMPLEMENTED");
-			
-		case SSH_MSG_DISCONNECT:
-			/* TODO cleanup? */
-			dropbear_close("Disconnect received");
+/* returns 1 if the packet is a valid type during kex (see 7.1 of rfc4253) */
+static int packet_is_okay_kex(unsigned char type) {
+	if (type >= SSH_MSG_USERAUTH_REQUEST) {
+		return 0;
 	}
-
-	/* Check if we should ignore this packet. Used currently only for
-	 * KEX code, with first_kex_packet_follows */
-	if (ses.ignorenext) {
-		TRACE(("Ignoring packet, type = %d", type));
-		ses.ignorenext = 0;
-		goto out;
+	if (type == SSH_MSG_SERVICE_REQUEST || type == SSH_MSG_SERVICE_ACCEPT) {
+		return 0;
 	}
-
-	/* check that we aren't expecting a particular packet */
-	if (ses.expecting && ses.expecting != type) {
-		/* TODO send disconnect? */
-		dropbear_exit("unexpected packet type %d, expected %d", type,
-				ses.expecting);
+	if (type == SSH_MSG_KEXINIT) {
+		/* XXX should this die horribly if !dataallowed ?? */
+		return 0;
 	}
+	return 1;
+}
 
-	/* handle the packet depending on type */
-	ses.expecting = 0;
-
-	switch (type) {
-
-		case SSH_MSG_SERVICE_REQUEST:
-			recv_msg_service_request();
-			break;
-
-		case SSH_MSG_USERAUTH_REQUEST:
-			recv_msg_userauth_request();
-			break;
-			
-		case SSH_MSG_KEXINIT:
-			recv_msg_kexinit();
-			break;
-
-		case SSH_MSG_KEXDH_INIT:
-			recv_msg_kexdh_init();
-			break;
-
-		case SSH_MSG_NEWKEYS:
-			recv_msg_newkeys();
-			break;
-
-		/* this is ugly, need to make a cleaner way to do it */
-		case SSH_MSG_CHANNEL_DATA:
-		case SSH_MSG_CHANNEL_WINDOW_ADJUST:
-		case SSH_MSG_CHANNEL_REQUEST:
-		case SSH_MSG_CHANNEL_OPEN:
-		case SSH_MSG_CHANNEL_EOF:
-		case SSH_MSG_CHANNEL_CLOSE:
-		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
-		case SSH_MSG_CHANNEL_OPEN_FAILURE:
-			/* these should be checked for authdone below */
-			process_postauth_packet(type);
-			break;
+static void enqueue_reply_packet() {
+	struct packetlist * new_item = NULL;
+	new_item = m_malloc(sizeof(struct packetlist));
+	new_item->next = NULL;
 	
-		default:
-			/* TODO this possibly should be handled */
-			TRACE(("unknown packet"));
-			recv_unimplemented();
-			break;
+	new_item->payload = buf_newcopy(ses.writepayload);
+	buf_setpos(ses.writepayload, 0);
+	buf_setlen(ses.writepayload, 0);
+	
+	if (ses.reply_queue_tail) {
+		ses.reply_queue_tail->next = new_item;
+	} else {
+		ses.reply_queue_head = new_item;
 	}
-
-out:
-	buf_free(ses.payload);
-	ses.payload = NULL;
-
-	TRACE(("leave process_packet"));
+	ses.reply_queue_tail = new_item;
+	TRACE(("leave enqueue_reply_packet"))
 }
 
-/* process a packet, and also check that auth has been done */
-static void process_postauth_packet(unsigned int type) {
-
-	/* messages following here require userauth before use */
-	if (!ses.authstate.authdone) {
-		dropbear_exit("received message %d before userauth", type);
+void maybe_flush_reply_queue() {
+	struct packetlist *tmp_item = NULL, *curr_item = NULL;
+	if (!ses.dataallowed)
+	{
+		TRACE(("maybe_empty_reply_queue - no data allowed"))
+		return;
 	}
-
-	switch (type) {
-
-		case SSH_MSG_CHANNEL_DATA:
-			recv_msg_channel_data();
-			break;
-
-		case SSH_MSG_CHANNEL_WINDOW_ADJUST:
-			recv_msg_channel_window_adjust();
-			break;
-
-		case SSH_MSG_CHANNEL_REQUEST:
-			recv_msg_channel_request();
-			break;
-
-		case SSH_MSG_CHANNEL_OPEN:
-			recv_msg_channel_open();
-			break;
-
-		case SSH_MSG_CHANNEL_EOF:
-			recv_msg_channel_eof();
-			break;
-
-		case SSH_MSG_CHANNEL_CLOSE:
-			recv_msg_channel_close();
-			break;
-
-#ifdef USE_LISTENERS /* for x11, tcp fwd etc */
-		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
-			recv_msg_channel_open_confirmation();
-			break;
+		
+	for (curr_item = ses.reply_queue_head; curr_item; ) {
+		CHECKCLEARTOWRITE();
+		buf_putbytes(ses.writepayload,
+			curr_item->payload->data, curr_item->payload->len);
 			
-		case SSH_MSG_CHANNEL_OPEN_FAILURE:
-			recv_msg_channel_open_failure();
-			break;
-#endif
-			
-		default:
-			TRACE(("unknown packet()"));
-			recv_unimplemented();
-			break;
+		buf_free(curr_item->payload);
+		tmp_item = curr_item;
+		curr_item = curr_item->next;
+		m_free(tmp_item);
+		encrypt_packet();
 	}
-}
-
-/* This must be called directly after receiving the unimplemented packet.
- * Isn't the most clean implementation, it relies on packet processing
- * occurring directly after decryption. This is reasonably valid, since
- * there is only a single decryption buffer */
-static void recv_unimplemented() {
-
-	CHECKCLEARTOWRITE();
-
-	buf_putbyte(ses.writepayload, SSH_MSG_UNIMPLEMENTED);
-	/* the decryption routine increments the sequence number, we must
-	 * decrement */
-	buf_putint(ses.writepayload, ses.recvseq - 1);
-
-	encrypt_packet();
+	ses.reply_queue_head = ses.reply_queue_tail = NULL;
 }
 	
-
-
 /* encrypt the writepayload, putting into writebuf, ready for write_packet()
  * to put on the wire */
 void encrypt_packet() {
@@ -575,9 +468,20 @@ void encrypt_packet() {
 	unsigned char blocksize, macsize;
 	buffer * writebuf; /* the packet which will go on the wire */
 	buffer * clearwritebuf; /* unencrypted, possibly compressed */
+	unsigned char type;
 	
-	TRACE(("enter encrypt_packet()"));
-	TRACE(("encrypt_packet type is %d", ses.writepayload->data[0]));
+	type = ses.writepayload->data[0];
+	TRACE(("enter encrypt_packet()"))
+	TRACE(("encrypt_packet type is %d", type))
+	
+	if (!ses.dataallowed && !packet_is_okay_kex(type)) {
+		/* During key exchange only particular packets are allowed.
+			Since this type isn't OK we just enqueue it to send 
+			after the KEX, see maybe_flush_reply_queue */
+		enqueue_reply_packet();
+		return;
+	}
+		
 	blocksize = ses.keys->trans_algo_crypt->blocksize;
 	macsize = ses.keys->trans_algo_mac->hashsize;
 
@@ -650,6 +554,7 @@ void encrypt_packet() {
 		while (clearwritebuf->pos < clearwritebuf->len) {
 			if (cbc_encrypt(buf_getptr(clearwritebuf, blocksize),
 						buf_getwriteptr(writebuf, blocksize),
+						blocksize,
 						&ses.keys->trans_symmetric_struct) != CRYPT_OK) {
 				dropbear_exit("error encrypting");
 			}
@@ -663,6 +568,7 @@ void encrypt_packet() {
 
 	/* clearwritebuf is finished with */
 	buf_free(clearwritebuf);
+	clearwritebuf = NULL;
 
 	/* enqueue the packet for sending */
 	buf_setpos(writebuf, 0);
@@ -672,22 +578,22 @@ void encrypt_packet() {
 	ses.kexstate.datatrans += writebuf->len;
 	ses.transseq++;
 
-	TRACE(("leave encrypt_packet()"));
+	TRACE(("leave encrypt_packet()"))
 }
 
 
 /* Create the packet mac, and append H(seqno|clearbuf) to the output */
 static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
 
-	int macsize;
+	unsigned int macsize;
 	unsigned char seqbuf[4];
-	unsigned long hashsize;
+	unsigned char tempbuf[MAX_MAC_LEN];
+	unsigned long bufsize;
 	hmac_state hmac;
 
-	TRACE(("enter writemac"));
+	TRACE(("enter writemac"))
 
 	macsize = ses.keys->trans_algo_mac->hashsize;
-
 	if (macsize > 0) {
 		/* calculate the mac */
 		if (hmac_init(&hmac, 
@@ -712,14 +618,14 @@ static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
 			dropbear_exit("HMAC error");
 		}
 	
-		hashsize = macsize;
-		if (hmac_done(&hmac, buf_getwriteptr(outputbuffer, macsize), &hashsize) 
+		bufsize = sizeof(tempbuf);
+		if (hmac_done(&hmac, tempbuf, &bufsize) 
 				!= CRYPT_OK) {
 			dropbear_exit("HMAC error");
 		}
-		buf_incrwritepos(outputbuffer, macsize);
+		buf_putbytes(outputbuffer, tempbuf, macsize);
 	}
-	TRACE(("leave writemac"));
+	TRACE(("leave writemac"))
 }
 
 #ifndef DISABLE_ZLIB
@@ -730,7 +636,7 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len) {
 	unsigned int endpos = src->pos + len;
 	int result;
 
-	TRACE(("enter buf_compress"));
+	TRACE(("enter buf_compress"))
 
 	while (1) {
 
@@ -756,7 +662,7 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len) {
 			break;
 		}
 
-		assert(ses.keys->trans_zstream->avail_out == 0);
+		dropbear_assert(ses.keys->trans_zstream->avail_out == 0);
 
 		/* the buffer has been filled, we must extend. This only happens in
 		 * unusual circumstances where the data grows in size after deflate(),
@@ -764,6 +670,6 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len) {
 		buf_resize(dest, dest->size + ZLIB_COMPRESS_INCR);
 
 	}
-	TRACE(("leave buf_compress"));
+	TRACE(("leave buf_compress"))
 }
 #endif
