@@ -15,7 +15,7 @@
  * called by a name other than "ssh" or "Secure Shell".
  */
 
-/*RCSID("$OpenBSD: sshpty.c,v 1.7 2002/06/24 17:57:20 deraadt Exp $");*/
+/*RCSID("OpenBSD: sshpty.c,v 1.7 2002/06/24 17:57:20 deraadt Exp ");*/
 
 #include "includes.h"
 #include "dbutil.h"
@@ -30,8 +30,8 @@
 #ifdef HAVE_PTY_H
 # include <pty.h>
 #endif
-#if defined(USE_DEV_PTMX) && defined(HAVE_SYS_STROPTS_H)
-# include <sys/stropts.h>
+#if defined(USE_DEV_PTMX) && defined(HAVE_STROPTS_H)
+# include <stropts.h>
 #endif
 
 #ifndef O_NOCTTY
